"""Actions that can be performed on instances and products.

Everything that is handy to have as a single unit of action is turned
into an Action. They can be quite small (just stopping zope), but
that's OK. Being able to say 'instancemanager myproject stop' from
anywhere without having to go to the instance directory to say
'bin/zopectl stop' saves on the number of terminals you have to keep
open.

The 'ACTIONS' dictionary at the end of this file is the list of
user-visible actions. There's no need for every action to be listed
there, so you can define Actions just for internal use, if you want.
"""

import config
import sys
import os
import os.path
import logging
import shutil
log = logging.getLogger('actions')


class Action:
    """Base class for all actions.
    """

    name = 'abstract base action'

    def __init__(self, configuration):
        self.conf = configuration

    def run(self):
        pass


class CreateInstanceAction(Action):
    """Create a zope instance.
    """

    name = "Create a zope instance for your project."

    def run(self):
        python = self.conf.configData['python']
        instanceDir = self.conf.instanceDir()
        if os.path.exists(instanceDir):
            shutil.rmtree(instanceDir)
            log.debug("Removed instance dir '%s'.", instanceDir)
        mkInstance = os.path.join(self.conf.zopeDir(),
                                  'bin',
                                  'mkzopeinstance.py')
        params = '-u %s:%s -d %s' % (
            self.conf.configData['user'],
            self.conf.configData['password'],
            instanceDir,
            )
        command = ' '.join([python, mkInstance, params])
        os.system(command)
        log.info("Created zope instance in '%s'.",
                 self.conf.instanceDir())
        # TODO: port handling, security mode, debug mode
        zopeConfigFile = os.path.join(instanceDir, 'etc', 'zope.conf')
        log.debug("Replacing parts of '%s'.", zopeConfigFile)
        oldFile = open(zopeConfigFile, 'r').readlines()
        newFile = open(zopeConfigFile, 'w')
        textsToChange = config.CONFIGCHANGES.keys()
        for line in oldFile:
            newLine = line
            for text in textsToChange:
                if text in line:
                    template = config.CONFIGCHANGES[text]
                    newLine = template % self.conf.configData
                    log.debug("Replaced %r", line)
                    log.debug("....with %r", newLine)
                    textsToChange.remove(text)
            newFile.write(newLine)
        newFile.close()
        log.info("Changed the zope config.")
                    

class CopyDatafsAction(Action):
    """Copy over a fresh, prepared, Data.fs.
    """

    name = "Copy over a fresh, prepared, 'Data.fs'."

    def run(self):
        source = self.conf.datafs()
        target = os.path.join(self.conf.instanceDir(),
                              'var',
                              'Data.fs')
        if not os.path.exists(source):
            log.info("No prepared Data.fs found at %s, "
                     "skipping this step.", source)
            return
        else:
            shutil.copy(source, target)
            log.info("Copied over fresh Data.fs from %s.", source)
            log.debug("Copied it to %s.", target)


class AddProductsAction(Action):
    """Rebuild the Products/ directory from scratch.

    Warning, this *does* remove everything in Products/. That is the
    whole purpose :-) If it isn't your purpose, you shouldn't really
    be using this program.
    """

    name = "Rebuild the Products/ directory ."

    def run(self):
        target = os.path.join(self.conf.instanceDir(), 'Products')
        shutil.rmtree(target)
        log.debug("Removed the entire %s directory.", target)
        os.mkdir(target)
        log.debug("Created it again.")
        log.debug("Adding Products...")
        for source in self.conf.sources():
            source.addProduct()
        log.info("Rebuild the Products/ directory from scratch.")


class FreshInstallAction(Action):
    """Wipe everything and create an instance including products.
    """

    name = "Wipe everything and create an instance " +\
        "including products."

    def run(self):
        subActions = [StopZopeAction,
                      CreateInstanceAction,
                      CopyDatafsAction,
                      AddProductsAction,
                      StartZopeAction,
                      QuickreinstallAction]
        for actionClass in subActions:
            action = actionClass(self.conf)
            action.run()

        
class SoftInstallAction(Action):
    """Restart zope and call the quickreinstall script.
    """

    name = "Restart zope and call the quickreinstall script."

    def run(self):
        subActions = [RestartZopeAction,
                      QuickreinstallAction,
                      ]
        for actionClass in subActions:
            action = actionClass(self.conf)
            action.run()

        
class BaseZopeAction(Action):
    """Base action for starting/stopping zope
    """

    name = "Base for the zope actions."
    argument = 'start'

    def run(self):
        zopectlCommand = os.path.join(self.conf.instanceDir(), 
                                      'bin',
                                      'zopectl')
        command = ' '.join([zopectlCommand, self.argument])
        os.system(command)


class StartZopeAction(BaseZopeAction):
    name = "Start the zope instance."
    argument = 'start'


class RestartZopeAction(BaseZopeAction):
    name = "Restart the zope instance."
    argument = 'restart'


class StopZopeAction(BaseZopeAction):
    name = "Stop the zope instance."
    argument = 'stop'


class QuickreinstallAction(Action):
    """Action that quickreinstalls your products.

    Depends on the presence of a specific script in your zope root
    that calls the actual quickinstaller. If the reinstall template
    gives an empty result, this action isn't performed.
    """

    name = "Action that quickreinstalls your products."

    def run(self):
        reinstallScript = self.conf.configData['reinstall_script']
        if reinstallScript:
            import time
            waitTime = self.conf.configData['zope_wait_time']
            log.info("Sleeping %s seconds.", waitTime)
            time.sleep(waitTime)
            wgetResultFile = reinstallScript
            if os.path.exists(wgetResultFile):
                os.remove(wgetResultFile)
                log.debug("Removed previous wget result file.")
            reinstall = self.conf.expandTemplate('reinstall_template')
            os.system(reinstall)
            log.info("Quick reinstall script called.")
        else:
            log.debug("No quick reinstall desired, apparently.")


# User-visible actions, this allows the user to do
# 'instancemanager yourproject fresh', for instance.
ACTIONS = {
    'create': CreateInstanceAction,
    'datafs': CopyDatafsAction,
    'products': AddProductsAction,
    'fresh': FreshInstallAction,
    'soft': SoftInstallAction,
    'start': StartZopeAction,
    'restart': RestartZopeAction,
    'stop': StopZopeAction,
    'reinstall': QuickreinstallAction,
    }
